/*
 * ChronosMEGA.asm
 * REV7
 *
 * Created: 3/29/2014 2:01:36 PM
 * Author: Nick
 *
 * Function:
 * Main source code, initializes variables and registers
 * Contains the Main Loop of the system
 * Also contains the ISRs
 */ 

 .include "displayLED.inc"
 .include "TimeLogic.inc"
 .include "ISRlogic.inc"

 .dseg		;initialize data segment, reserve spots in SRAM for data

	HR:			.BYTE 1				;data for Hour value
	MIN:		.BYTE 1				;data for Minute value
	SECND:		.BYTE 1				;data for Second value
	SEC10:		.BYTE 1				;data for one tenth of a second value
	dispBool:	.BYTE 1				;boolean value to display time or not
	slpBool:	.BYTE 1				;boolean value to put ChronosMEGA to sleep
	dispMIN:	.BYTE 1				;Hour data to actually be displayed
	dispHR:		.BYTE 1				;Minute data to actually be displayed
	idleArray:	.BYTE 10			;Maps SEC10 value to LED position

 .cseg	  ;Start code segment
	 ;Define addresses for Interupts
	 ;setup the RESET, External and Timer2 Interrupts
	.org	0x0000
		rjmp	RESET				;jmp to RESET on RESET interrupt

	.org	PCI1addr
		rjmp	ExInt1				;jmp to Ext1 on External 1 Interrupt

	.org	PCI2addr
		rjmp	ExInt2				;jmp to ExInt2 on External 2 Interrupt
		
	.org	OC2Aaddr
		rjmp	Timer2CompMatch		;jmp to Timer2CompMatch when Timer2 match is made
		
	.org	OC1Aaddr
		rjmp	Timer1CompMatch		;jmp to Timer1CompMatch when Timer1 match is made
 
	rjmp	RESET					;begin with RESET


/*
RESET
Here, all the constants will be initialized
*/
RESET:
	;Set up the stack pointer
	ldi		r16, high(RAMEND)
	out		SPH, r16
	ldi		r16, low(RAMEND)
	out		SPL, r16
	sei							;enable global interrupts

	ldi		r30, LOW(HR)
	ldi		r31, HIGH(HR)		;Setup Z to point to start of SRAM
	ldi		r16, 0x05			;load r16 with 5 (initial condition value, arbitrary)
	st		Z+, r16				;Store HR data with value in r16
	ldi		r16, 0x21			;load r16 with 33 (DEC 33 = HEX 18)
	st		Z+, r16				;Store MIN data with value in r16
	ldi		r16, 0x00			
	st		Z+, r16				;Store SEC data with value in r16
	st		Z+, r16				;Store SEC10
	ldi		r16, 0x00			;enable time set; set to 0x01 to display time and 0x00 to display idle
	st		Z+, r16				;Store slpBool
	st		Z+, r16				;Store slpBool, slpBool==0x00 => active
	ldi		r16, 0x00
	st		Z+, r16				;Store dispHR
	st		Z+, r16				;Store dispMIN
	
	;load the idleArrary into the SRAM, use the X register to point
	;the idleArray helps to map LEDs to certain 10th of a second values
	ldi		r26, low(idleArray)
	ldi		r27, high(idleArray)

	ldi		r16, 0x01
	st		X+, r16
	ldi		r16, 0x02
	st		X+, r16
	ldi		r16, 0x04
	st		X+, r16
	ldi		r16, 0x08
	st		X+, r16
	ldi		r16, 0x10
	st		X+, r16
	ldi		r16, 0x20
	st		X+, r16
	ldi		r16, 0x08
	st		X+, r16
	ldi		r16, 0x04
	st		X+, r16
	ldi		r16, 0x02
	st		X+, r16
	ldi		r16, 0x01
	st		X+, r16

	rjmp	RegInit			;Initialize the registers for HW

/*
RegInit
Initialize all HW registers that are to be used
*/
RegInit:
	;Set up Timer2 Registers for clocking
	ldi		r16, 0b00100000
	sts		ASSR, r16			;Setup for Async Timer; AVR runs off internal 8MHz
	ldi		r16, 0b00000010   
	sts		TCCR2A, r16			;Set Timer2 to Compare Match mode
	ldi		r16, 0x06
	sts		TCCR2B, r16			;Set clk prescaler to 256
	ldi		r16, 0x0C			;Load r16 with 128 (128x256 = 32768)
	sts		OCR2A, r16			;Set the Output Compare Reg to 101
	ldi		r16, 0b00000010
	sts		TIMSK2, r16			;Enable interrupts for the Output Compare Reg A
		
	;Setup Timer1 to count 5 seconds when "Time" button is pressed
	ldi		r16, 0b00001101
	sts		TCCR1B, r16			;Set Compare Match mode and prescaler to 1024
	ldi		r16, 0x12			
	sts		OCR1AH, r16			;The high byte must be written before low byte. See pg114 of datasheet
	ldi		r16, 0x13			;Set the OCR1A 16bit ret to 0x1312
	sts		OCR1AL, r16

	;Set up External Interupts
	ldi		r16, 0b00000110        
	sts		PCICR, r16			;Enable interupts for PCINT[23:8] pins
	sts		PCMSK2, r16			;Enable interrupts only for buttons 1 and 2
	ldi		r16, 0b00110000
	sts		PCMSK1, r16			;Enable interrupts only for buttons 3 and 4

	;Set up GPIO
	ldi		r16, 0b00000111
	out		DDRB, r16			;Make pins PB0, PB1 and PB2 outputs
	ldi		r16, 0b00001111
	out		DDRC, r16			;Make pins PC0, PC1, PC2 and PC3 outputs
	ldi		r16, 0b10111000	
	out		DDRD, r16			;Make pins PD3, PD4, PD5, PD6 and PD7 outputs

	;Set up Power Management and Sleep logic
	ldi		r16, 0b00000110
	out		SMCR, r16			;Set sleep mode to Power-save
	ldi		r16, 0xA2			
	sts		PRR, r16			;Shut off TWI, Timer0 and USART modules

MainLoop:
	RCALL	PrepDispVar				;Takes the variable values and sets up what should be displayed for Hours and Minutes
	RCALL	MapLEDs					;Maps what should be displayed to the LEDs on the GPIO
	rjmp	MainLoop_CheckStatus	;Check the sleep status

MainLoop_CheckStatus:
	lds		r16, slpBool			;load r16 with the slpBool
	SBRC	r16, 0					;Skip next instruction if 0th bit in slpBool is 0 (i.e. active state)
	rjmp	MainLoop_goToSleep		;Go to SLEEP if slpBool==0x01
	rjmp	MainLoop				;Repeat main loop if slpBool==0x00

MainLoop_goToSleep:
	in		r16, SMCR				;Load the SMCR register
	ori		r16, 0x01				;enable the SE bit in SMCR without affecting the other bits
	out		SMCR, r16				;reload the SMCR register
	SLEEP							;Go to sleep, program counter halts here
	;in		r16, SMCR				
	;andi	r16, 0xFE				;Clear the SE bit and reload it into SMCR register
	;out		SMCR, r16
	nop								
	rjmp	MainLoop_CheckStatus	;Check the slpBool again to see if system is still sleeping.

	
;Timer2CompMatch:
;When this interrupt is triggered, 1/10 of a second has passed
Timer2CompMatch:
	push	r16					;push R16 onto stack for later
	RCALL	IncSec10			;call the IncSec10 function and push address to stack
	pop		r16					;recover the original value for R16 
	reti						;pop address into program counter and enable interrupts

Timer1CompMatch:
	push	r16
	ldi		r16, 0x00	
	sts		TIMSK1, r16			;disable interrupts for Timer1
	sts		dispBool, r16		;set dispBool to show Idle
	pop		r16
	reti

ExInt1:
	push	r16
	push	r17
	RCALL	Button3or4logic
	pop		r17
	pop		r16
	reti

ExInt2:
	push	r16
	RCALL	Button1or2logic
	pop		r16
	reti
	