/*
 *		displayLED.inc
 *			An include file to the ChronosMEGA.asm
 *
 *		Function:
 *		Handles display processing, and LED mapping routines 
 *
 *  Created: 3/13/2014 9:47:22 PM
 *  Developer: Nick Williams
*/

.cseg
.org	0x0200

;PrepDispVar:
;There are two display modes:
;	Idle or Time
;	Compare the display boolean and figure out what mode
PrepDispVar:
	clz							;Clear the Z flag in CCR for comparison
	lds		r16, dispBool		;Load the dispBool
	cpi		r16, 0x00			;If dispBool == 0x00
	breq	IdleSet				;then branch to IdleSet
	cpi		r16, 0x01			;else if dispBool == 0x01
	breq	TimeSet				;then branch to TimeSet

;TimeSet:
;Loads the HR and MIN value directly into the dispMin and dispHr variables to
;show the Time
TimeSet:
	lds		r16, MIN
	lds		r17, HR
	sts		dispMin, r16		;Take MIN value and display it
	sts		dispHr,	r17			;Take HR value and display it
	ret

;IdleSet:
;This is relevant when the ChronosMEGA is in Idle mode
;The program takes the 1/10 of the second value (SEC10) and figures out which
;LED in the ring needs to be lit. Uses the idleArray
;r12 and r13 act as buffers to dispMin and dispHr respectively
IdleSet:									
	clz
	ldi		XH, HIGH(idleArray)	;point the X reg to the starting address of the idleArray
	ldi		XL, LOW(idleArray)
	lds		r17, SEC10			;load r17 with the SEC10 val
	ldi		r16, 0x00			;load r16 with 0x00, will act as the counter
	ld		r12, X+				;load r12 with the first value in the idleArray

IdleSet_loop:
	cp		r16, r17			;see if r16 matches r17
	breq	IdleSet_match		;branch if match is made
	inc		r16					;if not, increment the r16 register
	ld		r12, X+				;get the next value in the idleArray
	rjmp	IdleSet_loop		;loop

IdleSet_match:
	mov		r13, r12			;copy r12 to r13
	cpi		r17, 0x06			;if the SEC10 is less than 6
	brlt	IdleSet_clearR13	;then clear r13
	clr		r12					;otherwise clear r12
	rjmp	Idle_storeResult	

IdleSet_clearR13:
	clr		r13					;if SEC10 >= 6, clear r13
	rjmp	Idle_storeResult

Idle_storeResult:
	sts		dispHR, r13			;store r13 to dispHR
	sts		dispMIN, r12		;store r12 to dispMIN
	ret							;return

;MapLEDs
;maps the values of dispHR and dispMIN to the LEDs
;on the ChronosMEGA's face
MapLEDs:
	clr		r22
	clr		r23
	clr		r24
	
	lds		r13, dispMin
	lds		r12, dispHr

	clz
	lds		r16, slpBool	;check for sleep status
	SBRC	r16, 0
	RCALL	DisableLEDs

	SBRC	r13, 0			;Skip if bit 0 of Reg MIN is clear
	ORI		r23, 0x04		;Or PORTC with Immediate K: 0x04, i.e. turn on LED MIN1
	SBRC	r13, 1			;Skip if bit 1 of Reg MIN is clear
	ORI		r23, 0x02		;Or PORTC with Immediate K: 0x02, i.e. turn on LED MIN2
	SBRC	r13, 2			;Skip if bit 2 of Reg MIN is clear
	ORI		r23, 0x01		;Or PORTC with Immediate K: 0x01, i.e. turn on LED MIN4
	SBRC	r13, 3			;Skip if bit 3 of Reg MIN is clear
	ORI		r22, 0x02		;Or PORTB with Immediate K: 0x02, i.e. turn on LED MIN8
	SBRC	r13, 4			;Skip if bit 4 of Reg MIN is clear
	ORI		r22, 0x01		;Or PORTB with Immediate K: 0x01, i.e. turn on LED MIN16
	SBRC	r13, 5			;Skip if bit 5 of Reg MIN is clear
	ORI		r24, 0x80		;Or PORTD with Immediate K: 0x80, i.e. turn on LED MIN32
	SBRC	r12, 0			;Skip if bit 0 of Reg HOUR is clear
	ORI		r23, 0x08		;Or PORTC with Immediate K: 0x08, i.e. turn on LED HR1
	SBRC	r12, 1			;Skip if bit 1 of Reg HOUR is clear
	ORI		r24, 0x08		;Or PORTD with Immediate K: 0x08, i.e. turn on LED HR2
	SBRC	r12, 2			;Skip if bit 2 of Reg HOUR is clear
	ORI		r24, 0x10		;Or PORTD with Immediate K: 0x10, i.e. turn on LED HR4
	SBRC	r12, 3			;Skip if bit 3 of Reg HOUR is clear
	ORI		r24, 0x20		;Or PORTD with Immediate K: 0x20, i.e. turn on LED HR8

	OUT		PORTB, r22			;Load the I/O reg with R22
	OUT		PORTC, r23			;Load the I/O reg with R23
	OUT		PORTD, r24			;Load the I/O reg with R24

	ret 

DisableLEDs:
	clr		r13
	clr		r12
	ret